{% extends 'base.html' %}

{% block title %}q2-emperor : {{ plot_name }}{% endblock %}

{% block head %}

<style media="screen">
    body {
        overflow: hidden; /* hide rounding error in viewport calc */
    }

    iframe {
        height: 100vh;
        width: 100%;
        border: none;
    }

    #q2templatesheader ~ .row > iframe {
	height: calc(100vh - 85px);
    }
</style>

{% endblock %}

{% block content %}
    <div class="row text-center">
        <iframe src="./emperor.html"></iframe>
    </div>

{% endblock %}
